@@ -1,6 +1,6 @@
 /*  xfce4-appfinder
  *
- *  Copyright (C) 2004-2005 Eduard Roccatello (eduard@xfce.org)
+ *  Copyright (C) 2004-2006 Eduard Roccatello (eduard@xfce.org)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -27,8 +27,6 @@
 #include <libxfce4util/libxfce4util.h>
 #include <libxfcegui4/libxfcegui4.h>
 #include <string.h>
-
-#include "inline-icon.h"
 #include "xfce4-appfinder.h"
 
 enum {
@@ -99,8 +97,7 @@ static GHashTable *   createDesktopCache             ();
                                                
                                                                                               
 static gint           xfce_appfinder_signals[LAST_SIGNAL] = { 0 };
-static gchar        **desktop_entries_paths;
-static gint           desktop_path_number = 0;
+static GPtrArray *    desktop_entries_paths;
 static gint           showedcat = APPFINDER_ALL;
 
 /* What to search for in .desktop files */
@@ -140,7 +137,8 @@ static const GtkTargetEntry dotDesktopDropTarget[] =
     {"text/plain", 0, 1},
     {"application/x-desktop", 0, 2},
     {"STRING", 0, 3},
-    {"UTF8_STRING", 0, 4}
+    {"UTF8_STRING", 0, 4},
+    {"text/uri-list", 0, 5}
 };
 
 GType
@@ -222,7 +220,7 @@ xfce_appfinder_init (XfceAppfinder *appfinder)
     appfinder->appsTree = create_applications_treeview(appfinder);
     gtk_tree_view_enable_model_drag_source( GTK_TREE_VIEW(appfinder->appsTree),
                                             GDK_BUTTON1_MASK, dotDesktopDropTarget,
-                                            5, GDK_ACTION_COPY);
+                                            6, GDK_ACTION_COPY);
 
     g_signal_connect(appfinder->appsTree, "row-activated", G_CALLBACK(callbackApplicationActivate), (gpointer) appfinder);
     g_signal_connect(appfinder->appsTree, "button-press-event", G_CALLBACK(callbackApplicationRightClick), (gpointer) appfinder);    
@@ -379,8 +377,10 @@ xfce_appfinder_list_add (gchar *name, XfceAppfinderCacheEntry *entry, XfceAppfin
     
     if (param->psearch)
     {
-        if (!(entry->comment && g_pattern_match_string (param->psearch, g_utf8_casefold(entry->comment, -1))) &&
-                    !g_pattern_match_string (param->psearch, name))
+        if (!((entry->comment && g_pattern_match_string (param->psearch, g_utf8_casefold(entry->comment, -1))) ||
+               (entry->exec && g_pattern_match_string (param->psearch, g_utf8_casefold(entry->exec, -1))) ||
+                g_pattern_match_string (param->psearch, g_utf8_casefold(name, -1))
+              ))
         {
             return;
         }
@@ -416,7 +416,7 @@ load_desktop_resources (gint category, gchar *pattern, XfceAppfinder *appfinder)
 
     if (pattern != NULL)
     {
-        tmp = g_strconcat("*", g_utf8_casefold(pattern, -1), "*", NULL);
+        tmp = g_strconcat("*", pattern, "*", NULL);
         psearch = g_pattern_spec_new (tmp);
         g_free(tmp);
     }
@@ -460,53 +460,47 @@ build_desktop_paths (void)
     gint    napps;
     gchar **applnk;
     gint    napplnk;
-    gint    i, n;
+    gint    n;
     
     applications = xfce_resource_lookup_all (XFCE_RESOURCE_DATA, "applications/");
-    for (napplications = 0; applications[napplications] != NULL; ++napplications);
     
     apps = xfce_resource_lookup_all (XFCE_RESOURCE_DATA, "apps/");
-    for (napps = 0; apps[napps] != NULL; ++napps);
     
     applnk = xfce_resource_lookup_all (XFCE_RESOURCE_DATA, "applnk/");
-    for (napplnk = 0; applnk[napplnk] != NULL; ++napplnk);
     
-    desktop_entries_paths = g_new0 (gchar *, 2 * napplications + napps + napplnk + 6);
-    i = 0;
+    desktop_entries_paths = g_ptr_array_new ();
     
-    desktop_entries_paths[i++] = xfce_get_homefile (".gnome", "share", "apps", NULL);
-    desktop_entries_paths[i++] = xfce_get_homefile (".kde", "share", "applnk", NULL);
+    g_ptr_array_add (desktop_entries_paths, (gpointer) xfce_get_homefile (".gnome", "share", "apps", NULL));
+    g_ptr_array_add (desktop_entries_paths,  (gpointer) xfce_get_homefile (".kde", "share", "applnk", NULL));
     if ((kdedir = g_getenv("KDEDIR")) != NULL)
     {
-        desktop_entries_paths[i++] = g_build_filename (kdedir, "share",  "applications",  "kde", NULL); 
+        g_ptr_array_add (desktop_entries_paths, (gpointer) g_build_filename (kdedir, "share",  "applications",  "kde", NULL)); 
     }
 
     /* FreeBSD Gnome stuff */
-    desktop_entries_paths[i++] = g_build_filename ("/usr", "X11R6", "share", "gnome", "applications", NULL);
+    g_ptr_array_add (desktop_entries_paths, (gpointer) g_build_filename ("/usr", "X11R6", "share", "gnome", "applications", NULL));
 
     /* /usr/global stuff */
-    desktop_entries_paths[i++] = g_build_filename ("/usr", "global", "share", "applications", NULL);
+    g_ptr_array_add (desktop_entries_paths, (gpointer) g_build_filename ("/usr", "global", "share", "applications", NULL));
     
-    for (n = 0; n < napplications; ++n)
+    for (n = 0; applications[n] != NULL; ++n)
     {
-        desktop_entries_paths[i++] = applications[n];
-        desktop_entries_paths[i++] = g_build_filename (applications[n], "kde", NULL);
+        g_ptr_array_add (desktop_entries_paths, (gpointer) applications[n]);
+        g_ptr_array_add (desktop_entries_paths, (gpointer) g_build_filename (applications[n], "kde", NULL));
     }
     g_free (applications);
     
-    for (n = 0; n < napps; ++i, ++n)
+    for (n = 0; apps[n] != NULL; ++n)
     {
-        desktop_entries_paths[i] = apps[n];
+        g_ptr_array_add (desktop_entries_paths, (gpointer) apps[n]);
     }
     g_free (apps);
     
-    for (n = 0; n < napplnk; ++i, ++n)
+    for (n = 0; applnk[n] != NULL; ++n)
     {
-        desktop_entries_paths[i] = applnk[n];
+        g_ptr_array_add (desktop_entries_paths, (gpointer) applnk[n]);
     }
     g_free (applnk);
-    
-    desktop_path_number = n;
 }
 
 static gchar *get_path_from_name(gchar *name, XfceAppfinder *appfinder)
@@ -579,7 +573,7 @@ callbackCategoryTreeClick  (GtkTreeSelection *selection,
         {
             gtk_list_store_append(GTK_LIST_STORE(gtk_tree_view_get_model (GTK_TREE_VIEW(af->appsTree))), &iter);
             gtk_list_store_set(GTK_LIST_STORE(gtk_tree_view_get_model (GTK_TREE_VIEW(af->appsTree))),
-                                &iter, APPLICATION_TREE_ICON, xfce_inline_icon_at_size (default_icon_data_48_48, 24, 24),
+                                &iter, APPLICATION_TREE_ICON, xfce_themed_icon_load("xfce4-appfinder", 24),
                                 APPLICATION_TREE_TEXT, _("No items available"), -1);
             gtk_widget_set_sensitive(af->appsTree, FALSE);
         }
@@ -598,7 +592,7 @@ xfce_appfinder_search (XfceAppfinder *appfinder, const gchar *pattern)
     GtkTreeSortable   *sortable;
     GtkListStore      *liststore;    
     GtkTreeIter iter;
-    gchar *text = g_utf8_strdown(pattern, -1);
+    gchar *text = g_utf8_casefold(pattern, -1);
     showedcat = APPFINDER_ALL;
     
     liststore = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(appfinder->appsTree)));
@@ -619,7 +613,7 @@ xfce_appfinder_search (XfceAppfinder *appfinder, const gchar *pattern)
     {
         gtk_list_store_append(GTK_LIST_STORE(gtk_tree_view_get_model (GTK_TREE_VIEW(appfinder->appsTree))), &iter);
         gtk_list_store_set(GTK_LIST_STORE(gtk_tree_view_get_model (GTK_TREE_VIEW(appfinder->appsTree))),
-                            &iter, APPLICATION_TREE_ICON, xfce_inline_icon_at_size (default_icon_data_48_48, 24, 24),
+                            &iter, APPLICATION_TREE_ICON, xfce_themed_icon_load("xfce4-appfinder", 24),
                             APPLICATION_TREE_TEXT, _("Sorry, no match for searched text."), -1);
         gtk_widget_set_sensitive(appfinder->appsTree, FALSE);
     }
@@ -744,18 +738,17 @@ createDesktopCache()
     gchar                    *filename  = NULL;
     gchar                    *fullpath  = NULL;
     GDir                     *dir;
-    gint                      n         = desktop_path_number - 1;
     gint                      i         = 0; /* A counter for general use */
     
     hash = g_hash_table_new ((GHashFunc) g_str_hash, (GEqualFunc) g_str_equal);
     
-    while (desktop_entries_paths[i]!=NULL)
+    while (i<desktop_entries_paths->len)
     {
-        if ((dir = g_dir_open (desktop_entries_paths[i], 0, NULL))!=NULL)
+        if ((dir = g_dir_open ((gchar *) g_ptr_array_index(desktop_entries_paths,i), 0, NULL))!=NULL)
         {
             while ((filename = (gchar *)g_dir_read_name(dir))!=NULL)
             {
-                fullpath = g_build_filename(desktop_entries_paths[i], filename, NULL);
+                fullpath = g_build_filename(g_ptr_array_index(desktop_entries_paths,i), filename, NULL);
                 if (g_str_has_suffix(filename, ".desktop"))
                 {
                     dentry = xfce_desktop_entry_new (fullpath, dotDesktopKeys, 7);
@@ -781,13 +774,30 @@ createDesktopCache()
                                 cent->categories = g_strdup(categories);
                                 cent->path = g_strdup(fullpath);
                                 cent->exec = g_strdup(exec);
-                                cent->comment = g_strdup(comment);
-                                cent->icon = g_strdup(icon);
+
+				if (comment)
+				{
+                                    cent->comment = g_strdup(comment);
+				}
+				else
+				{
+				    cent->comment = NULL;
+				}
+				
+				if (icon)
+			        {
+                                    cent->icon = g_strdup(icon);
+				}
+				else
+				{
+				   cent->icon = NULL;
+				}
+				
                                 g_hash_table_insert(hash, cent->name, cent);
                                 
-                                g_free(name);
-                                g_free(categories);
-                                g_free(exec);
+                                    g_free(name);
+                                    g_free(categories);
+                                    g_free(exec);
                                 if (comment)
                                     g_free(comment);
                                 if (icon)
@@ -800,8 +810,7 @@ createDesktopCache()
                 }
                 else if (g_file_test(fullpath, G_FILE_TEST_IS_DIR))
                 {
-                    desktop_entries_paths[n] = fullpath;
-                    desktop_entries_paths[n] = NULL;
+                    g_ptr_array_add(desktop_entries_paths, (gpointer) fullpath);
                 }
             }
             g_dir_close(dir);
